#ifndef BONJOURREGISTRAR_H
#define BONJOURREGISTRAR_H

#include <QObject>
#include "bonjourrecord.h"

#undef USE_ZEROCONF

#ifdef Q_OS_LINUX
#include <avahi-compat-libdns_sd/dns_sd.h>
#define USE_ZEROCONF
#elif defined(Q_OS_MAC)
//#define USE_ZEROCONF
#endif

class QSocketNotifier;
#ifdef USE_ZEROCONF
typedef DNSServiceErrorType ErrorType;
#else
typedef int ErrorType;
#endif

class BonjourRegistrar : public QObject
{
	Q_OBJECT

public:
	BonjourRegistrar(QObject *parent = 0);
	~BonjourRegistrar();

	void registerService(const BonjourRecord &record, quint16 servicePort);
	const BonjourRecord &registeredRecord() const;

signals:
	void error(ErrorType error);
	void serviceRegistered(const BonjourRecord &record);

private slots:
	void bonjourSocketReadyRead();

private:
#ifdef USE_ZEROCONF
	static void DNSSD_API bonjourRegisterService(DNSServiceRef,
												 DNSServiceFlags,
												 DNSServiceErrorType,
												 const char *,
												 const char *,
												 const char *,
												 void *);

#endif

private:
#ifdef USE_ZEROCONF
	DNSServiceRef dnssref;
#endif
	BonjourRecord finalRecord;
	QSocketNotifier *bonjourSocket;
};

#endif // BONJOURREGISTRAR_H
