#include "computer.h"
#include <QDesktopServices>
#include <QUrl>
#include <QTime>
#include <QDataStream>

#define TIME_OUT_DELAY	20000

Computer::Computer() : hostname(), port(2000), name()
{
	time = new QTime;
	time->start();
}

Computer::Computer(const QByteArray &data)
{
	time = new QTime;
	time->start();
	unserialize(data);
}

Computer::Computer(const Computer &c) : QObject(), hostname(c.hostname), port(c.port), name(c.name)
{
	time = new QTime;
	*time = *c.time;
}

Computer &Computer::operator=(const Computer &c)
{
	hostname = c.hostname;
	name = c.name;
	port = c.port;
	*time = *c.time;
	return *this;
}

bool Computer::operator!=(const Computer &c)
{
	return c.port != port || c.name != name || c.hostname != hostname;
}

void Computer::open(const QString &path) const
{
	QDesktopServices::openUrl(QUrl("ftp://" + hostname + ":" + QString::number(port) + path));
}

QString Computer::getHostname() const
{
	return hostname;
}

QString Computer::getName() const
{
	return name;
}

int Computer::getPort() const
{
	return port;
}

void Computer::setPort(const int port)
{
	this->port = port;
	time->start();
}

void Computer::setHostname(const QString &hostname)
{
	this->hostname = hostname;
	time->start();
}

void Computer::setName(const QString &name)
{
	this->name = name;
	time->start();
}

bool operator!=(const Computer &a, const Computer &b)
{
	return a.getHostname() != b.getHostname() || a.getName() != b.getName() || a.getPort() != b.getPort();
}

bool Computer::timedOut() const
{
	return time->elapsed() > TIME_OUT_DELAY;
}

QByteArray Computer::serialize() const
{
	QByteArray data;
	QDataStream stream(&data, QIODevice::WriteOnly);
	stream  << hostname << port << name;
	return data;
}

void Computer::unserialize(const QByteArray &data)
{
	QDataStream stream(data);
	stream >> hostname >> port >> name;
}
