#ifndef COMPUTER_H
#define COMPUTER_H

#include <QObject>
#include <QByteArray>
#include <QDataStream>

class QTime;

class Computer : public QObject
{
	Q_OBJECT
public:
    Computer();
	Computer(const QByteArray &data);
	Computer(const Computer &c);
	Computer &operator=(const Computer &c);
	bool operator!=(const Computer &c);

	QString getHostname() const;
	QString getName() const;
	int getPort() const;
	bool timedOut() const;

	QByteArray serialize() const;
	void unserialize(const QByteArray &data);

public slots:
	void open(const QString &path = QString()) const;
	void setHostname(const QString &hostname);
	void setName(const QString &name);
	void setPort(const int port);

private:
	QString hostname;
	int port;
	QString name;
	QTime *time;
};

bool operator!=(const Computer &a, const Computer &b);

inline QDataStream &operator<<(QDataStream &out, const Computer &in)
{
	return out << in.serialize();
}

inline QDataStream &operator>>(QDataStream &in, Computer &out)
{
	QByteArray data;
	in >> data;
	out.unserialize(data);
	return in;
}

#endif // COMPUTER_H
