#include "ipcsocket.h"
#include "ipcconnection.h"
#include <QLocalSocket>
#include <QLocalServer>

IPCSocket::IPCSocket(QObject *parent) : QObject(parent)
{
	sock = NULL;
	connect(this,SIGNAL(disconnecting()),this,SLOT(resetState()));
}

IPCSocket::IPCSocket(const QString &name, QObject *parent) : QObject(parent)
{
	sock = NULL;
	connect(this,SIGNAL(disconnecting()),this,SLOT(resetState()));
	connectTo(name);
}

void IPCSocket::connectTo(const QString &name)
{
	emit disconnecting();

	sock = NULL;
	currentState = NotConnected;
	QLocalSocket *s = new QLocalSocket(this);
	connect(this,SIGNAL(disconnecting()),s,SLOT(deleteLater()));
	connect(s,SIGNAL(error(QLocalSocket::LocalSocketError)),this,SLOT(resetState()));
	connect(s,SIGNAL(destroyed()),this,SLOT(resetState()));
	s->connectToServer(name);
	if (s->waitForConnected(2000))
	{
		IPCConnection *ipc = new IPCConnection(s);
		connect(ipc,SIGNAL(message(QByteArray)),this,SLOT(dispatchMessage(QByteArray)));
		connect(this,SIGNAL(sigSendMessage(QByteArray)),ipc,SLOT(sendMessage(QByteArray)));
		relayConnectedSignal();
	}
	else			// No server to connect to, let's create one
	{
		s->deleteLater();
		sock = new QLocalServer(this);
		connect(sock,SIGNAL(newConnection()),this,SLOT(processConnection()));
		connect(this,SIGNAL(disconnecting()),sock,SLOT(deleteLater()));
		if (sock->listen(name))
		{
			connect(sock,SIGNAL(destroyed()),this,SLOT(resetState()));
			currentState = Server;
			emit connected();
		}
	}
}

void IPCSocket::processConnection()
{
	if (!sock)
		return;

	while(sock->hasPendingConnections())
	{
		QLocalSocket *s = sock->nextPendingConnection();
		IPCConnection *ipc = new IPCConnection(s);
		connect(ipc,SIGNAL(message(QByteArray)),this,SLOT(dispatchMessage(QByteArray)));
		connect(this,SIGNAL(sigSendMessage(QByteArray)),ipc,SLOT(sendMessage(QByteArray)));
	}
}

void IPCSocket::dispatchMessage(const QByteArray &msg)
{
	emit message(msg);
}

void IPCSocket::sendMessage(const QByteArray &msg)
{
	emit sigSendMessage(msg);
}

IPCSocket::State IPCSocket::state()
{
	return currentState;
}

void IPCSocket::resetState()
{
	currentState = NotConnected;
}

void IPCSocket::relayConnectedSignal()
{
	currentState = Client;
	emit connected();
}

IPCSocket::~IPCSocket()
{
}
