#include "ipfile.h"
#include "userspath.h"
#include "ipfileserver.h"
#include "path.h"
#include "ftpserver.h"
#include <QStringList>
#include "delay.h"

IPFile::IPFile(const UserID &uid, const QString &filename) : fileID(0), bOpen(false), uid(uid)
{
	QStringList path = filename.split(Path::separator, QString::SkipEmptyParts);
	if (path.isEmpty())
	{
		this->filename.clear();
		this->user.clear();
		return;
	}
	const QSet<QString> &users = FtpServer::instance()->getUsers();
	if (!users.contains(path[0]))
	{
		this->filename.clear();
		this->user.clear();
		return;
	}
	user = path[0];
	path.removeFirst();
	this->filename = Path::separator + path.join(Path::separator);
	fullname = filename;
}

IPFile::~IPFile()
{
	close();
}

bool IPFile::exists() const
{
	if (user.isEmpty())
		return false;
	return UsersPath::exists(uid, fullname);
}

bool IPFile::isReadable() const
{
	if (!bOpen)
		return false;
	return IPFileServer::instance()->isReadable(user, fileID);
}

bool IPFile::isWritable() const
{
	if (!bOpen)
		return false;
	return IPFileServer::instance()->isWritable(user, fileID);
}

qint64 IPFile::size() const
{
	if (user.isEmpty())
		return 0;
	return UsersPath::size(uid, fullname);
}

bool IPFile::isOpen() const
{
	return bOpen;
}

bool IPFile::seek(qint64 offset)
{
	if (!bOpen)
		return true;
	return IPFileServer::instance()->seek(user, fileID, offset);
}

void IPFile::open(QFile::OpenMode mode)
{
	if (user.isEmpty())
		return;
	if (bOpen)
		close();
	bOpen = true;
	fileID = IPFileServer::instance()->open(uid, user, filename, mode);
}

void IPFile::close()
{
	if (bOpen)
	{
		bOpen = false;
		IPFileServer::instance()->close(user, fileID);
	}
}

void IPFile::flush()
{
	if (bOpen)
		IPFileServer::instance()->flush(user, fileID);
}

QByteArray IPFile::read(qint64 maxlen)
{
	if (!bOpen)
		return QByteArray();
	return IPFileServer::instance()->read(user, fileID, maxlen);
}

void IPFile::write(const QByteArray &data)
{
	if (bOpen)
		IPFileServer::instance()->write(user, fileID, data);
}
