#ifndef NETDISCOVERY_H
#define NETDISCOVERY_H

#include <QRunnable>
#include <QMap>
#include <QMutex>
#include "computer.h"
#include "ipobject.h"

class QHostAddress;
class BonjourRegistrar;

class NetDiscovery : public IPObject, public QRunnable
{
	Q_OBJECT
private:
    NetDiscovery();
	virtual ~NetDiscovery();

public:
	void run();

	QList<Computer> getServerList();
	Computer getComputer(const QString &name);

	bool isRunning();

protected:
	virtual void receive(const QByteArray &message);
	bool hasToStop();

public slots:
	void setOrUpdate(const QString &data, const QHostAddress &host);
	void terminate();
	void forceRefresh();
	void remove(const QHostAddress &host);

signals:
	void serverListChanged();

private slots:
	void propagateServerListChanged();
	void start();

private:
	bool bRefreshForced;
	QMutex mutex;
	bool bStop;
	bool bRunning;
	bool bSendNotifyRequest;
	QMap<QString, Computer>	mServer;
	BonjourRegistrar *registrar;

private:
	static NetDiscovery *pInstance;
public:
	static NetDiscovery *instance();
};

#endif // NETDISCOVERY_H
